/**
 *    @file  suite_libhttp.c
 *   @brief  Check testsuite for libhttp functions
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-22 08:26
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include "checktools.inc"

#include "libhttp.h"
#include <stdio.h>                              /* perror */

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(libhttp_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

START_TEST(libhttp_httpheader_new_null) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;

    HTTPHeader_new(NULL,NULL);
}
END_TEST

START_TEST(libhttp_httpheader_del_null) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;

    HTTPHeader_del(NULL);
}
END_TEST

START_TEST(libhttp_httpheader_newdel) {
    HTTPHeader_t* header=NULL;

    header = HTTPHeader_new("name","value");
    ck_assert(header);
    ck_assert(0==strcmp("name",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("value",HTTPHeader_getvalue(header)));
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_httpheader_newdel_empty) {
    HTTPHeader_t* header=NULL;

    header = HTTPHeader_new("","");
    ck_assert(header);
    ck_assert(0==strcmp("",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("",HTTPHeader_getvalue(header)));
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_httpheader_newdel_doublefree) {
    HTTPHeader_t* header=NULL;

    /* Always SIG11 (not caught by assertions) */
    forktest_only;

    header = HTTPHeader_new("","");
    ck_assert(header);
    ck_assert(0==strcmp("",HTTPHeader_getname(header)));
    ck_assert(0==strcmp("",HTTPHeader_getvalue(header)));
    HTTPHeader_del(header);
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_httpheader_basicauth) {
    HTTPHeader_t* header=NULL;

    header = HTTPHeader_basicauth("admin@demo.com","password");
    ck_assert(0==strcmp("Authorization",HTTPHeader_getname(header)));
    /* Could be dynamically created with base64.header/base64.c inclusion */
    ck_assert(0==strcmp("Basic YWRtaW5AZGVtby5jb206cGFzc3dvcmQ=",HTTPHeader_getvalue(header)));
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_newdel) {
    HTTP_t* uut;

    uut = HTTP_new();
    HTTP_del(uut);
}
END_TEST

START_TEST(libhttp_newdel_doublefree) {
    HTTP_t* uut;

    /* Always SIG11 (not caught by assertions) */
    forktest_only;

    uut = HTTP_new();
    HTTP_del(uut);

    /* Double free should trigger SIG11 */
    HTTP_del(uut);
}
END_TEST

START_TEST(libhttp_del_null) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;

    HTTP_del(NULL);
}
END_TEST

START_TEST(libhttp_setbody_null_empty) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;
    HTTP_setbody(NULL,"");
}
END_TEST

START_TEST(libhttp_setbody_null) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;
    HTTP_t* uut;
    uut = HTTP_new();
    HTTP_setbody(uut,NULL);
}
END_TEST

START_TEST(libhttp_setbody_empty) {
    HTTP_t* uut;
    uut = HTTP_new();
    HTTP_setbody(uut,"");
    ck_assert(0==strcmp("",HTTP_getbody(uut)));
    HTTP_del(uut);
}
END_TEST

START_TEST(libhttp_setbody_valid) {
    HTTP_t* uut;
    uut = HTTP_new();
    HTTP_setbody(uut,"abc");
    ck_assert(0==strcmp("abc",HTTP_getbody(uut)));
    HTTP_del(uut);
}
END_TEST

START_TEST(libhttp_getbody_first) {
    HTTP_t* uut;
    uut = HTTP_new();
    ck_assert(0==strcmp("",HTTP_getbody(uut)));
    HTTP_del(uut);
}
END_TEST

START_TEST(libhttp_addheader_null_null) {
    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;
    HTTP_addheader(NULL,NULL);
}
END_TEST

START_TEST(libhttp_addheader_null) {
    HTTP_t* uut;

    /* SIG11 in release build, SIG6(assert/abort) in debug build */
    forktest_only;

    uut = HTTP_new();
    HTTP_addheader(uut,NULL);
}
END_TEST

START_TEST(libhttp_addheader_valid) {
    HTTP_t* uut;
    HTTPHeader_t* header;

    uut = HTTP_new();
    header = HTTPHeader_new("name","value");
    ck_assert(NULL!=HTTP_addheader(uut,header));
    HTTP_delheader(uut,header);
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_addheader_doubleadd) {
    HTTP_t* uut;
    HTTPHeader_t* header;

    uut = HTTP_new();
    header = HTTPHeader_new("name","value");
    ck_assert(NULL!=HTTP_addheader(uut,header));
    ck_assert(NULL==HTTP_addheader(uut,header));
    HTTP_delheader(uut,header);
    HTTPHeader_del(header);
}
END_TEST

START_TEST(libhttp_basictest) {
    char* tmp;
    HTTP_t* uut;
    HTTPHeader_t* header;

    uut = HTTP_new();

    HTTP_setbody(uut,"<HTML></HTML>\r\n");

    if (NULL==(header=HTTPHeader_new("Host","localhost"))) {
        perror("libhttp_basictest HTTPHeader_new Host");
        ck_abort();
    }
    if (NULL==(uut=HTTP_addheader(uut,header))) {
        perror("libhttp_basictest HTTP_addheader Host");
        ck_abort();
    }

    if (NULL==(header=HTTPHeader_new("Host","localhost"))) {
        perror("libhttp_basictest HTTPHeader_new Host2");
        ck_abort();
    }
    if (NULL==(uut=HTTP_addheader(uut,header))) {
        perror("libhttp_basictest HTTP_addheader Host2");
        ck_abort();
    }

    if (NULL==(header=HTTPHeader_new("Content-type","text/html"))) {
        perror("libhttp_basictest HTTPHeader_new Host");
        ck_abort();
    }
    if (NULL==(uut=HTTP_addheader(uut,header))) {
        perror("libhttp_basictest HTTP_addheader Host");
        ck_abort();
    }

    tmp=HTTP_buildrequest(HTTPMETHOD_GET,"/v1/api/bdbs",HTTPVERSION_HTTP11);
    printf("%s",tmp);
    free(tmp);

    tmp = HTTP_buildheaders(uut);
    printf("%s",tmp);
    free(tmp);

    printf("%s",HTTP_getbody(uut));

    HTTP_del(uut);

}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void libhttp_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void libhttp_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void libhttp_unchecked_common_setup() {
    forktest_init();
}
void libhttp_unchecked_common_teardown() {
}
/************************************************************************/
Suite* libhttp_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("LibHTTP");

    tc = tcase_create("LibHTTPUninitialized");
    tcase_set_tags(tc,"LibHTTPTag LibHTTPTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, libhttp_checked_uninitialized_setup, libhttp_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, libhttp_unchecked_common_setup, libhttp_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, libhttp_test,6);
    tcase_add_test_raise_signal(tc, libhttp_httpheader_new_null,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_httpheader_del_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_httpheader_newdel);
    tcase_add_test(tc, libhttp_httpheader_newdel_empty);
    tcase_add_test_raise_signal(tc, libhttp_httpheader_newdel_doublefree,11);
    tcase_add_test(tc, libhttp_httpheader_basicauth);
    tcase_add_test(tc, libhttp_newdel);
    tcase_add_test_raise_signal(tc, libhttp_newdel_doublefree,11);
    tcase_add_test_raise_signal(tc, libhttp_del_null,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_setbody_null_empty,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_setbody_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_setbody_empty);
    tcase_add_test(tc, libhttp_setbody_valid);
    tcase_add_test(tc, libhttp_getbody_first);
    tcase_add_test_raise_signal(tc, libhttp_addheader_null_null,SIG_ABRTSEGV);
    tcase_add_test_raise_signal(tc, libhttp_addheader_null,SIG_ABRTSEGV);
    tcase_add_test(tc, libhttp_addheader_valid);
    tcase_add_test(tc, libhttp_addheader_doubleadd);
    /*
     * libhttp_firstheader
     * libhttp_nextheader
     * libhttp_findheader
     * libhttp_delheader
     */
    tcase_add_test(tc, libhttp_basictest);

    return s;
}
/* vim: set tw=80: */
